/* 
   Rombank (Java Edition) - A imbNES rombank.bin generator for multiple platforms
   Author: Bruno Freitas (aka Sparrow) - 03/2004
   Contact: bootsector@ig.com.br
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.io.*;

/** This class generate the contents of the rombank file, just after the TOC. The
 * content is represented by the ROMs data plus some padding bytes, for the
 * sectors.
 */
public class Contents {
   final int sectorSize = 0x800; // 2048 decimal
   
   private RomsList romsList;   
   
   /** This is the class constructor! It requires a RomsList as parameter.
    * @param roms The RomsList which will provide all the necessary data for the contents
    * generating.
    */   
   public Contents(RomsList roms) {
      romsList = roms;
   }
   
   /** Write the contents to the disk.  Note that this must be called before
    * TOC.writeTOC, ok!!! :-)
    * @param filename The file which the data should be written on! This must be the same file used by
    * TOC.writeTOC!!
    */   
   public void writeContents(String filename) {
      if (romsList.romsCount() < 1)
         return;
      
      byte romContents[];

      try {
         RandomAccessFile outfile = new RandomAccessFile(filename, "rw");
         
         // Seeks EOF
         outfile.seek(outfile.length());
         
         for(int i = 0; i < romsList.romsCount(); i++) {
            romContents = new byte[(int)romsList.romSize(i)];
            romsList.romReadData(i, romContents);
            outfile.write(romContents);

            // Pad the sector...
            int secSize = (int)Math.ceil((double)romsList.romSize(i) / sectorSize) * sectorSize;
            int padSize = secSize - (int)romsList.romSize(i);
         
            for(int j = 1; j <= padSize; j++)
               outfile.write(0x00);
            
            System.out.print("\r" + i * 100 / romsList.romsCount() + "% Completed...");
         }

         System.out.println("\r100% Completed! ");
         
         // Closes file
         outfile.close();
      } catch (Exception e) {
         System.err.println("An error has occurred while saving file " + filename);
      }
   }
}
